#! /usr/bin/perl

# Takes in a metacall file and exports the gains/loss/complex coordinates
# Optionally correct for maq's bug that shifts all coordinates by 1kb

use strict;
use Getopt::Long;
use IntSpan;

my $correct_factor = 0;
my $window = 1000;
my $bCorrect = 0;
my $bHelp = 0;

GetOptions("window" => \$window,
			"correct" => \$bCorrect,
			"help" => \$bHelp);

if($bHelp)
{
	print "extractCNChanges.pl [OPTIONS] [FILE]\n";
	print "extractCNChanges.pl - parse the metacall FILE (or stdin) and export the gains/losses\n";
	print "	--help	Print this message and exit\n";
	print "	--window=INT	Use INT as the window size (the parameter that was given to maq cns2win)\n";
	print "	--correct	Enable correction for the off-by-one bug in maq cns2win\n";
	exit(0);
}

my @stateList = ("G", "L"); # We care about gains, losses and complex
my $stateFilter = "[" . join("", @stateList) . "]";
my $t_factor = 1000000;
my $half_window = $window / 2;

my $spans;

# Read in the coords
while(<>)
{
	chomp;
	my ($chr, $s, $e, $state) = split(/\t/);
	if($state =~ /$stateFilter/)
	{
		# Coordinates are given by the midpoint of the range, convert to the enclosing coords
		my $start = parseAndCorrectCoord($s) - $half_window;
		my $end = parseAndCorrectCoord($e) + $half_window;

		if(!defined($spans->{$chr}->{$state}))
		{
			$spans->{$chr}->{$state} = Set::IntSpan->new();
		}
		$spans->{$chr}->{$state} += "$start-$end";
	}
}

foreach my $chr (keys %{$spans})
{
	foreach my $s (@stateList)
	{
		my @runs = split(/,/,$spans->{$chr}->{$s}->run_list);
		foreach my $r (@runs)
		{
			my ($start, $end) = split(/-/, $r);
			print "$chr\t$start\t$end\t$s\n";
		}
	}
}


sub parseAndCorrectCoord
{
	my ($c) = @_;
	#Translate 
	$c *= $t_factor;

	# Correct
	if($bCorrect)
	{
		$c -= $window;
	}

	return $c;
}



