//-----------------------------------------------
// Copyright 2009 Wellcome Trust Sanger Institute
// Written by Jared Simpson (js18@sanger.ac.uk)
// Released under the GPL license
//-----------------------------------------------
import Alphabet;
import Distribution;
import State;
import MarkovModel;
import HmmAlgorithms;
import CNVModel;
import tango.io.Stdout;
import Integer = tango.text.convert.Integer;
import Float = tango.text.convert.Float;
import tango.io.stream.FileStream;
import tango.text.Util;
import tango.io.stream.LineStream;
import tango.util.ArgParser;
import tango.math.Math;
import tango.util.collection.ArraySeq;

class JointSymbol : Symbol
{
	this(RealSymbol hs, RealSymbol ds, RealSymbol hitsym)
	{
		hetSymbol = hs;
		depthSymbol = ds;
		hitSymbol = hitsym;
	}

	char[] toString() { return hetSymbol.toString() ~ "," ~ depthSymbol.toString() ~ "," ~ hitSymbol.toString(); }


	RealSymbol hetSymbol;
	RealSymbol depthSymbol;
	RealSymbol hitSymbol;
}

//
// The JointCNVDistribution is the triplet of values (het rate, depth, number of hits) emitted by the model
// The three values are assumed to be independent
//
class JointCNVDistribution : IDistribution
{
	this(IDistribution hetD, IDistribution depthD, IDistribution hitD)
	{
		m_hetDist = hetD;
		m_depthDist = depthD;
		m_hitDist = hitD;
	}

	// Get the weight of the given symbol (from the PDF or PMF)
	real getWeight(Symbol s)
	{
		if(s.classinfo != JointSymbol.classinfo)
		{
			throw new Exception("Invalid symbol type to JointCNVDistribution: " ~ s.classinfo.name);
		}
		
		JointSymbol js = cast(JointSymbol)s;
		// Todo: calculate in logspace?
		return (m_hetDist.getWeight(js.hetSymbol) * m_depthDist.getWeight(js.depthSymbol) * m_hitDist.getWeight(js.hitSymbol));
	}

	// Get the probability (from the CDF) that X is less than x
	real getProbLessThan(Symbol s)
	{
		if(s.classinfo != JointSymbol.classinfo)
		{
			throw new Exception("Invalid symbol type to JointCNVDistribution: " ~ s.classinfo.name);
		}

		JointSymbol js = cast(JointSymbol)s;
		// Todo: calculate in logspace?
		return (m_hetDist.getProbLessThan(js.hetSymbol) * m_depthDist.getProbLessThan(js.depthSymbol) * m_hitDist.getProbLessThan(js.hitSymbol));
		
	}

	void printWeight(Symbol s)
	{
		if(s.classinfo != JointSymbol.classinfo)
		{
			throw new Exception("Invalid symbol type to JointCNVDistribution: " ~ s.classinfo.name);
		}
		
		JointSymbol js = cast(JointSymbol)s;
		real h_w = m_hetDist.getWeight(js.hetSymbol);
		real d_w = m_depthDist.getWeight(js.depthSymbol);
		real r_w = m_hitDist.getWeight(js.hitSymbol);

		Stdout.format("Emission Probs [{0} -> {1:F6}] [{2} -> {3:F6}] [{4} -> {5:F6}]\n",
					  js.depthSymbol.toString(), d_w, js.hetSymbol.toString(), h_w, js.hitSymbol.toString(), r_w);
	}

	// Sample a symbol from the distribution
	Symbol sample()
	{
		throw new Exception("Sampling not implemented for joint cnv");
		return new Symbol("");		
	}

	// Does the distribution contain this symbol
	bool contains(Symbol s)
	{
		return true;
	}

	IDistribution m_hetDist;
	IDistribution m_depthDist;
	IDistribution m_hitDist;
}	

